@@ -9,7 +9,6 @@ import threading
 import typing as t
 from abc import ABCMeta
 from functools import update_wrapper
-from types import AsyncGeneratorType, TracebackType
 
 from sniffio import thread_local as sniffio_loop
 
@@ -20,9 +19,12 @@ T = t.TypeVar("T")
 RetT = t.TypeVar("RetT")
 
 if t.TYPE_CHECKING:
-    from typing_extensions import ParamSpec, Self
+    from types import AsyncGeneratorType, TracebackType
+
+    from typing_extensions import ParamSpec, Self, TypeVarTuple, Unpack
 
     ArgsT = ParamSpec("ArgsT")
+    PosArgsT = TypeVarTuple("PosArgsT")
 
 
 if t.TYPE_CHECKING:
@@ -101,9 +103,9 @@ def is_main_thread() -> bool:
 # Call the function and get the coroutine object, while giving helpful
 # errors for common mistakes. Returns coroutine object.
 ######
-# TODO: Use TypeVarTuple here.
 def coroutine_or_error(
-    async_fn: t.Callable[..., t.Awaitable[RetT]], *args: t.Any
+    async_fn: t.Callable[[Unpack[PosArgsT]], t.Awaitable[RetT]],
+    *args: Unpack[PosArgsT],
 ) -> collections.abc.Coroutine[object, t.NoReturn, RetT]:
     def _return_value_looks_like_wrong_library(value: object) -> bool:
         # Returned by legacy @asyncio.coroutine functions, which includes
