@@ -1,10 +1,12 @@
+from __future__ import annotations
+
 import sys
 from typing import TYPE_CHECKING
 
 from . import _core
 from ._abc import ReceiveStream, SendStream
 from ._core._windows_cffi import _handle, kernel32, raise_winerror
-from ._util import ConflictDetector, Final
+from ._util import ConflictDetector, final
 
 assert sys.platform == "win32" or not TYPE_CHECKING
 
@@ -23,10 +25,10 @@ class _HandleHolder:
         _core.register_with_iocp(self.handle)
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return self.handle == -1
 
-    def close(self):
+    def close(self) -> None:
         if self.closed:
             return
         handle = self.handle
@@ -34,11 +36,12 @@ class _HandleHolder:
         if not kernel32.CloseHandle(_handle(handle)):
             raise_winerror()
 
-    def __del__(self):
+    def __del__(self) -> None:
         self.close()
 
 
-class PipeSendStream(SendStream, metaclass=Final):
+@final
+class PipeSendStream(SendStream):
     """Represents a send stream over a Windows named pipe that has been
     opened in OVERLAPPED mode.
     """
@@ -49,7 +52,7 @@ class PipeSendStream(SendStream, metaclass=Final):
             "another task is currently using this pipe"
         )
 
-    async def send_all(self, data: bytes):
+    async def send_all(self, data: bytes) -> None:
         with self._conflict_detector:
             if self._handle_holder.closed:
                 raise _core.ClosedResourceError("this pipe is already closed")
@@ -75,15 +78,16 @@ class PipeSendStream(SendStream, metaclass=Final):
             # not implemented yet, and probably not needed
             await _core.checkpoint()
 
-    def close(self):
+    def close(self) -> None:
         self._handle_holder.close()
 
-    async def aclose(self):
+    async def aclose(self) -> None:
         self.close()
         await _core.checkpoint()
 
 
-class PipeReceiveStream(ReceiveStream, metaclass=Final):
+@final
+class PipeReceiveStream(ReceiveStream):
     """Represents a receive stream over an os.pipe object."""
 
     def __init__(self, handle: int) -> None:
@@ -92,7 +96,7 @@ class PipeReceiveStream(ReceiveStream, metaclass=Final):
             "another task is currently using this pipe"
         )
 
-    async def receive_some(self, max_bytes=None) -> bytes:
+    async def receive_some(self, max_bytes: int | None = None) -> bytes:
         with self._conflict_detector:
             if self._handle_holder.closed:
                 raise _core.ClosedResourceError("this pipe is already closed")
@@ -131,9 +135,9 @@ class PipeReceiveStream(ReceiveStream, metaclass=Final):
                 del buffer[size:]
                 return buffer
 
-    def close(self):
+    def close(self) -> None:
         self._handle_holder.close()
 
-    async def aclose(self):
+    async def aclose(self) -> None:
         self.close()
         await _core.checkpoint()
